--[[
Spezialisierung zum rendern der Anzahl von verbleibenden Ballen für Folie und Garn auf den Bildschirm.

Author:	Ifko[nator]
Datum:	22.07.2016

Version:	v3.5

History:	v1.0 @ 18.09.2015 - initial implementation
			-------------------------------------------------------------------------------------------------------------------------
			v2.0 @ 29.09.2015 - the timing of the net wrap consumption has been adjusted, little bit dirty but it works
							  -	set correct rotation for the bale in the bale chamber --## removed in v3.5!
							  - bales will now added to the ingame stats, also the already pressed bales --## removed in v3.5!
			-------------------------------------------------------------------------------------------------------------------------
			v3.0 @ 03.10.2015 - set correct rotation for the middle net roll --## removed in v3.5!
							  - animated pickup roll when baler is filling --## removed in v3.5!
							  - bug fix for the case, the main chamber door is opening and the wrapper table is not rotated forwards --## removed in v3.5!
							  - bug fix for load the wrong bale size, little bit dirty but it works --## removed in v3.5!
			-------------------------------------------------------------------------------------------------------------------------
			v3.5 @ 22.07.2016 - support for the V2 of the Krone Ultima and removed some fixes, they are not longer needed here
]]

KroneUltimaExtension = {};

local kroneUltimaExtension_directory = g_currentModDirectory;

local modDesc = loadXMLFile("modDesc", kroneUltimaExtension_directory .. "modDesc.xml");
local version = Utils.getNoNil(getXMLString(modDesc, "modDesc.version"), "!!! --->>> Missing 'version' attribute in modDesc file! <<<--- !!!");

print("### Krone Ultima Extension by Ifko[nator] in der Version: '" .. version .. "' erfolgreich geladen!");

delete(modDesc);

function KroneUltimaExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Ultima, specializations);
end;

function KroneUltimaExtension:load(xmlFile)
	if g_currentMission.ultimaExtensionCompatibleToV3 ~= nil then
		--## Info for BM Modding: Set 'g_currentMission.ultimaExtensionCompatibleToV3' in your Ultima.lua to false, to simply Stop my Script, if it makes Problems with an newer Version of your Baler.
		return;
	end;
	
	if not g_currentMission.ultimaExtensionCompatibleToV2 then
		g_currentMission.ultimaExtensionCompatibleToV2 = "Hell yeah! Thanks to BM-Modding for adding this variable in the Ultima.lua!";
	end;
	
	local modDesc = loadXMLFile("modDesc", kroneUltimaExtension_directory .. "modDesc.xml");
	local allowWrapStrawBales = Utils.getNoNil(getXMLBool(modDesc, "modDesc.ultima#allowWrapStrawBales"), true);
	
	if allowWrapStrawBales then
		local wheat_windrow = Fillable.fillTypeNameToInt["wheat_windrow"];
		local barley_windrow = Fillable.fillTypeNameToInt["barley_windrow"];
		local grass_windrow = Fillable.fillTypeNameToInt["grass_windrow"];
		
		local fileNameForWrapping130 = Utils.getFilename("bales/Bale130StrawWrap.i3d", kroneUltimaExtension_directory);
		local fileNameForWrapping140 = Utils.getFilename("bales/Bale140StrawWrap.i3d", kroneUltimaExtension_directory);
		local fileNameForWrapping150 = Utils.getFilename("bales/Bale150StrawWrap.i3d", kroneUltimaExtension_directory);
	
		if (fileNameForWrapping130 and fileNameForWrapping140 and fileNameForWrapping150) ~= nil then
			self.baleSizes[1].fillTypes[wheat_windrow].isWrapAble = true;
			self.baleSizes[2].fillTypes[wheat_windrow].isWrapAble = true;
			self.baleSizes[3].fillTypes[wheat_windrow].isWrapAble = true;
			
			self.baleSizes[1].fillTypes[wheat_windrow].fileNameForWrapping = fileNameForWrapping130;
			self.baleSizes[2].fillTypes[wheat_windrow].fileNameForWrapping = fileNameForWrapping140;
			self.baleSizes[3].fillTypes[wheat_windrow].fileNameForWrapping = fileNameForWrapping150;
			
			self.baleSizes[1].fillTypes[wheat_windrow].wrappedFileName = self.baleSizes[1].fillTypes[grass_windrow].wrappedFileName;
			self.baleSizes[2].fillTypes[wheat_windrow].wrappedFileName = self.baleSizes[2].fillTypes[grass_windrow].wrappedFileName;
			self.baleSizes[3].fillTypes[wheat_windrow].wrappedFileName = self.baleSizes[3].fillTypes[grass_windrow].wrappedFileName;
			
			self.baleSizes[1].fillTypes[barley_windrow].isWrapAble = true;
			self.baleSizes[2].fillTypes[barley_windrow].isWrapAble = true;
			self.baleSizes[3].fillTypes[barley_windrow].isWrapAble = true;
			
			self.baleSizes[1].fillTypes[barley_windrow].fileNameForWrapping = fileNameForWrapping130;
			self.baleSizes[2].fillTypes[barley_windrow].fileNameForWrapping = fileNameForWrapping140;
			self.baleSizes[3].fillTypes[barley_windrow].fileNameForWrapping = fileNameForWrapping150;
			
			self.baleSizes[1].fillTypes[barley_windrow].wrappedFileName = self.baleSizes[1].fillTypes[grass_windrow].wrappedFileName;
			self.baleSizes[2].fillTypes[barley_windrow].wrappedFileName = self.baleSizes[2].fillTypes[grass_windrow].wrappedFileName;
			self.baleSizes[3].fillTypes[barley_windrow].wrappedFileName = self.baleSizes[3].fillTypes[grass_windrow].wrappedFileName;
		else
			print("[ERROR from the KroneUltimaExtension.lua]: Cant load Bales for Straw wrapping!");
		end;
	end;
	
	self.hudPosYCamOff = Utils.getNoNil(getXMLString(modDesc, "modDesc.ultima#hudPosYCamOff"), 0.356);
	self.hudPosYCamOn = Utils.getNoNil(getXMLString(modDesc, "modDesc.ultima#hudPosYCamOn"), 0.53);
	self.hudPosY = self.hudPosYCamOff;
	self.hudPosYHideGUI = g_currentMission.vehicleHudPosY + g_currentMission.vehicleBaseHudHeight + g_currentMission.hudBackgroundHeight;
	
	delete(modDesc);
	
	self.hudWidth = 0.0335;
	self.hudHeight = 0.045;
	
	self.hudFoilPosX = 0.826;
	self.hudNetPosX = 0.915;
	
	self.hudOverlayFoilFilename = Utils.getFilename("baleFoil.dds", kroneUltimaExtension_directory);
	self.hudOverlayFoilCamOffOverlay = Overlay:new("hudOverlayFoilCamOffOverlay", self.hudOverlayFoilFilename, self.hudFoilPosX, self.hudPosYCamOff, self.hudWidth, self.hudHeight);
	self.hudOverlayFoilCamOnOverlay = Overlay:new("hudOverlayFoilCamOnOverlay", self.hudOverlayFoilFilename, self.hudFoilPosX, self.hudPosYCamOn, self.hudWidth, self.hudHeight);
	self.hudOverlayFoilHideGUIOverlay = Overlay:new("hudOverlayFoilHideGUIOverlay", self.hudOverlayFoilFilename, self.hudFoilPosX, self.hudPosYHideGUI, self.hudWidth, self.hudHeight);
	
	self.hudOverlayNetFilename = Utils.getFilename("baleNet.dds", kroneUltimaExtension_directory);
	self.hudOverlayNetCamOffOverlay = Overlay:new("hudOverlayNetCamOffOverlay", self.hudOverlayNetFilename, self.hudNetPosX, self.hudPosYCamOff, self.hudWidth, self.hudHeight);
	self.hudOverlayNetCamOnOverlay = Overlay:new("hudOverlayNetCamOnOverlay", self.hudOverlayNetFilename, self.hudNetPosX, self.hudPosYCamOn, self.hudWidth, self.hudHeight);
	self.hudOverlayNetHideGUIOverlay = Overlay:new("hudOverlayNetHideGUIOverlay", self.hudOverlayNetFilename, self.hudNetPosX, self.hudPosYHideGUI, self.hudWidth, self.hudHeight);
	
	self.hudBackgroundWidth = g_currentMission.hudBackgroundWidth + 1;
	
	self.hudOverlayBackgroundFilename = Utils.getFilename("background.dds", kroneUltimaExtension_directory);
	self.hudOverlayBackgroundCamOff = Overlay:new("hudOverlayBackgroundCamOff", self.hudOverlayBackgroundFilename, self.hudFoilPosX, self.hudPosYCamOff, self.hudBackgroundWidth, self.hudHeight);
	self.hudOverlayBackgroundCamOn = Overlay:new("hudOverlayBackgroundCamOn", self.hudOverlayBackgroundFilename, self.hudFoilPosX, self.hudPosYCamOn, self.hudBackgroundWidth, self.hudHeight);
	self.hudOverlayBackgroundHideGUI = Overlay:new("hudOverlayBackgroundHideGUI", self.hudOverlayBackgroundFilename, self.hudFoilPosX, self.hudPosYHideGUI, self.hudBackgroundWidth, self.hudHeight);

	self.netBindingStart = Utils.appendedFunction(self.netBindingStart, KroneUltimaExtension.netBindingStart);
	self.dropBaleStart = Utils.appendedFunction(self.dropBaleStart, KroneUltimaExtension.dropBaleStart);
	
	self.netPerBale = 0;
	
	self.showFoilAndNetHud = true;
	
	for _, workArea in pairs(self.workAreas) do
		--print("--## set Tanslation for workArea start");
		setTranslation(workArea.start, 1.5, 0, 0);
		--print("--## set Tanslation for workArea width");
		setTranslation(workArea.width, -1.5, 0, 0);
		--print("--## set Tanslation for workArea height");
		setTranslation(workArea.height, 1.5, 0, -0.75);
	end;
end;

function KroneUltimaExtension:netBindingStart()
	--print("--## using: 'KroneUltimaExtension:netBindingStart'.");
	
	--## dirty hack do stop the count down of the net wrap bale count when the net bindig starts .. (simply add the net to the net role that was removed from the Ultima.lua)
	if self.isServer then
		if self.netRoleTop.length > 0 then
			self:setNetRoleFillState(self.netRoleTop.length + self.netPerBale);
		end;
	end;
end;

function KroneUltimaExtension:dropBaleStart()
	--print("--## using: 'KroneUltimaExtension:dropBaleStart'.");
	if self.isServer then
		--## now, when the net binding is finished, the counter for the net wrap bales will be set down ..
		self:setNetRoleFillState(self.netRoleTop.length - self.netPerBale);
	end;
end;

function KroneUltimaExtension:update(dt)
	if g_currentMission.ultimaExtensionCompatibleToV3 ~= nil then
		return;
	end;
	
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.TOGGLE_FOIL_AND_NET_COUNTER_HUD) then
			self.showFoilAndNetHud = not self.showFoilAndNetHud;
		end;
	end;
end;

function KroneUltimaExtension:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if g_currentMission.ultimaExtensionCompatibleToV3 ~= nil then
		return nil;
	end;
	
	if not resetVehicles then
		self.showFoilAndNetHud = Utils.getNoNil(getXMLBool(xmlFile, key .. "#showFoilAndNetHud"), self.showFoilAndNetHud);
	end;

	return BaseMission.VEHICLE_LOAD_OK;
end;

function KroneUltimaExtension:getSaveAttributesAndNodes(nodeIdent)
    if g_currentMission.ultimaExtensionCompatibleToV3 ~= nil then
		return nil, nil;
	end;
	
	local attributes = 'showFoilAndNetHud="'.. tostring(self.showFoilAndNetHud) .. '"';

	return attributes, nil;
end;

function KroneUltimaExtension:draw()
	if g_currentMission.ultimaExtensionCompatibleToV3 ~= nil then
		g_currentMission:showBlinkingWarning(Fillable.UltimaExtensionNotCompatibleToV3, 1);
		
		return;
	end;
	
	if self:getIsActive() then
		if self.showFoilAndNetHud then
			g_currentMission:addHelpButtonText(Fillable.HIDE_FOIL_AND_NET_COUNTER_HUD, InputBinding.TOGGLE_FOIL_AND_NET_COUNTER_HUD);
			
			if not self.hud.hideGUI then	
				if self.workCamera ~= nil and self.workCamera.baseElement ~= nil then
					if self.workCamera.baseElement.visibility then
						self.hudOverlayBackgroundCamOn:render();
						self.hudOverlayFoilCamOnOverlay:render();
						self.hudOverlayNetCamOnOverlay:render();
						
						self.hudPosY = self.hudPosYCamOn;
					else
						self.hudOverlayBackgroundCamOff:render();
						self.hudOverlayFoilCamOffOverlay:render();
						self.hudOverlayNetCamOffOverlay:render();
					
						self.hudPosY = self.hudPosYCamOff;
					end;
				end;
			else
				self.hudOverlayBackgroundHideGUI:render();
				self.hudOverlayFoilHideGUIOverlay:render();
				self.hudOverlayNetHideGUIOverlay:render();
			
				self.hudPosY = self.hudPosYHideGUI;
			end;
		
			setTextAlignment(RenderText.ALIGN_CENTER);
			
			local foilLength = self.wrapperFoilHolders[1].remainingFoilLength * 2;
			local foilPerBale = self.baseFoilUsePerBale * self.baleSizes[self.baleSizeIndex].size;		
			local numLeftBalesFoil = math.ceil(foilLength / foilPerBale);
			
			local textSize = getCorrectTextSize(0.028);
			local offsetX = 0.06;
			local offsetY = 0.014;
			
			if numLeftBalesFoil <= 5 then
				setTextColor(0.95, 0, 0, 1);
			else
				setTextColor(1, 1, 1, 1);
			end;
	
			renderText(self.hudFoilPosX + offsetX, self.hudPosY + offsetY, textSize, tostring(numLeftBalesFoil));
	
			self.netPerBale = math.max(1, self.baleSizes[self.baleSizeIndex].size * math.pi * self.numNetWraps);
			
			local numLeftBalesNet = math.floor(self.netRoleTop.length / self.netPerBale);
			
			if numLeftBalesNet <= 5 then
				setTextColor(0.95, 0, 0, 1);
			else
				setTextColor(1, 1, 1, 1);
			end;
			
			setVisibility(self.netRoleTop.role, numLeftBalesNet > 0);
			
			renderText(self.hudNetPosX + offsetX, self.hudPosY + offsetY, textSize, tostring(numLeftBalesNet));
		else
			g_currentMission:addHelpButtonText(Fillable.SHOW_FOIL_AND_NET_COUNTER_HUD, InputBinding.TOGGLE_FOIL_AND_NET_COUNTER_HUD);
		end;
	end;
	
	setTextAlignment(RenderText.ALIGN_LEFT);
	setTextColor(1, 1, 1, 1);
end;

function KroneUltimaExtension:delete()end;
function KroneUltimaExtension:mouseEvent(posX, posY, isDown, isUp, button)end;
function KroneUltimaExtension:keyEvent(unicode, sym, modifier, isDown)end;